/*****************************************************************************
 * Open MCT Web, Copyright (c) 2014-2015, United States Government
 * as represented by the Administrator of the National Aeronautics and Space
 * Administration. All rights reserved.
 *
 * Open MCT Web is licensed under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations
 * under the License.
 *
 * Open MCT Web includes source code licensed under additional open source
 * licenses. See the Open Source Licenses file (LICENSES.md) included with
 * this source code distribution or the Licensing information page available
 * at runtime from the About dialog for additional information.
 *****************************************************************************/

/*global define */
define(
    function () {
        "use strict";


        /**
         * The CopyAction is available from context menus and allows a user to
         * deep copy an object to another location of their choosing.
         *
         * @implements Action
         */
        function CopyAction(locationService, copyService, context) {

            var object,
                newParent,
                currentParent;

            if (context.selectedObject) {
                newParent = context.domainObject;
                object = context.selectedObject;
            } else {
                object = context.domainObject;
            }

            currentParent = object
                .getCapability('context')
                .getParent();

            return {
                perform: function () {

                    if (newParent) {
                        return copyService
                            .perform(object, newParent);
                    }

                    var dialogTitle,
                        label,
                        validateLocation;

                    dialogTitle = [
                        "Duplicate ",
                        object.getModel().name,
                        " to a location"
                    ].join("");

                    label = "Duplicate To";

                    validateLocation = function (newParent) {
                        return copyService
                            .validate(object, newParent);
                    };

                    return locationService.getLocationFromUser(
                        dialogTitle,
                        label,
                        validateLocation,
                        currentParent
                    ).then(function (newParent) {
                        return copyService
                            .perform(object, newParent);
                    });
                }
            };
        }

        return CopyAction;
    }
);
